structure DeflateList :> DeflateList =
struct

	fun sorted ordered [] = true
	  | sorted ordered (x::xs) =
		let 
			fun f (x,SOME max) = if ordered (max, x) then SOME x else NONE
			  | f _ = NONE
		in 
			isSome (foldl f (SOME x) xs)
		end

	fun sort _ [] = []
	  | sort _ (x::[]) = [x]
	  | sort ordered xs = 
		let
			fun merge xs [] = xs
			  | merge [] ys = ys
			  | merge (x::xs) (y::ys) = 
  				if ordered (x, y) then 
					x::merge xs (y::ys)
				else
					y::merge (x::xs) ys
			val split = (xs, (length xs) div 2)
		in
			merge (sort ordered (List.take split)) (sort ordered (List.drop split))
		end

	fun enum min max = 
		if min <= max then
			min :: enum (min + 1) max
		else
			[]
	
	fun replicate times value =
	  	if times > 0 then
			value::(replicate (times - 1) value)
		else
			[]

	fun state_map fs initstate =
		let
			fun impl [] (result, state) = (rev result, state)
			  | impl (f::fs) (result, state) =
				let
					val (value, next) = f state
				in
					impl fs (value :: result, next)
				end
		in
			impl fs ([], initstate)
		end

	(* self tests *)
	val tests = [
		sort op< [3,2,1] = [1,2,3],
		sort op< [1,2,3] = [1,2,3],
		sort op< [2] = [2],
		sort op< [] = [],
		sorted op< [1,2,3],
		sorted op< [2,1,3] = false,
		sorted op< [1,3,2] = false,
		sorted op< [],
		enum 1 4 = [1,2,3,4],
		enum 2 2 = [2],
		enum 1 0 = [],
		replicate 0 "a" = [],
		replicate 1 "b" = ["b"],
		replicate 3 "a" = ["a","a","a"],
		state_map [] [1,2,3,4] = ([],[1,2,3,4]),
		state_map (replicate 2 (fn xs => (3*hd xs, tl xs))) [1,2,3,4] = ([3,6],[3,4])]
				  	
end
