structure DeflateOutput :> DeflateOutput =
struct

	type out = (int * DeflateHuffman.tree) * int list
		(* the huffman tree is used as a generic binary tree here *)
		
	val buffer_size_power = 15 
		(* circular tree buffer has 2^15 entries, max backward distance in Deflate *)
		
	val buffer_size = DeflateBits.pow2 buffer_size_power

	val explode_byte = DeflateBits.explode_fixed_msb_first buffer_size_power

	val empty = ((0,DeflateHuffman.none), [])
	
	fun output (_, list) = rev list
	
	fun append_byte ((pos,tree),list) byte =
		let
			val newpos = (pos + 1) mod buffer_size
		in
			((newpos, DeflateHuffman.force tree (explode_byte newpos) byte), byte::list)
		end
			
	fun append_bytes output [] = output
	  | append_bytes output (b::bytes) = 
			append_bytes (append_byte output b) bytes

	fun get_back ((pos,tree),_) distance =
		Option.valOf (DeflateHuffman.code tree (explode_byte ((pos + 1 - distance) mod buffer_size)))
		
	fun append_from_output output 0 _ = output
	  | append_from_output output nbytes distance =
	  		append_from_output 
				(append_byte output (get_back output distance))
				(nbytes - 1) 
				distance
	
	(* self test *)
	val tests = [
		get_back (append_byte empty 111) 1 = 111]
end
